// 
// Copyright (C) 2011, MidasExplorer.com
// MidasStochObv is an implementation of the MIDAS Stoch-OBV created by Andrew Coles, 
// published in the book "Midas Technical Analysis" by Andrew Coles and David Hawkins, 2011.
//
// This implementation is heavily based on the standard indicators in NinjaTrader, which are:
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
//


#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Stochastics of OBV, used for MIDAS analysis.  Created by MidasExplorer.com
    /// </summary>
    [Description("Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.")]
    public class MidasStochObv : Indicator
    {
        #region Variables
		
		private int				periodK	= 3;	// Kperiod
		private int				smooth	= 3;	// SlowKperiod
		private int				periodD	= 7;	// SlowDperiod
		private DataSeries		den;
		private DataSeries		nom;
		private DataSeries		obv;

		#endregion

        protected override void Initialize()
        {
			Add(new Plot(Color.Orange, "K"));
			Add(new Plot(Color.Transparent, "D"));

			Add(new Line(Color.DarkViolet, 10, "Lower"));
			Add(new Line(Color.YellowGreen, 90, "Upper"));

			den			= new DataSeries(this);
			nom			= new DataSeries(this);
			obv			= new DataSeries(this);
        }

        protected override void OnBarUpdate()
        {
			if (CurrentBar == 0)
				obv.Set(0);
			else
			{
				if (Close[0] > Close[1])
					obv.Set(obv[1]+ Volume[0]);
				else if (Close[0]  < Close[1])
					obv.Set(obv[1] - Volume[0]);
				else
					obv.Set(obv[1]);
			}

            nom.Set(obv[0] - MIN(obv, PeriodK)[0]);
            den.Set(MAX(obv, PeriodK)[0] - MIN(obv, PeriodK)[0]);

            double nomSum = SUM(nom, Smooth)[0];
            double denSum = SUM(den, Smooth)[0];

            nomSum = nomSum.Compare(0, 0.000000000001) == 0 ? 0 : nomSum;
            denSum = denSum.Compare(0, 0.000000000001) == 0 ? 0 : denSum;

			if (denSum == 0) 
                K.Set(CurrentBar == 0 ? 50 : K[1]); 
            else
                K.Set(Math.Min(100, 100 * nomSum / denSum));

            D.Set(SMA(K, PeriodD)[0]);        
		}

		
        #region Properties

		/// <summary>
		/// Gets the slow D value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries D
		{
			get { return Values[0]; }
		}


		/// <summary>
		/// Gets the slow K value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries K
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for moving average over K values")]
		[GridCategory("Parameters")]
		public int PeriodD
		{
			get { return periodD; }
			set { periodD = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculating the K values")]
		[GridCategory("Parameters")]
		public int PeriodK
		{
			get { return periodK; }
			set { periodK = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing the slow K values")]
		[GridCategory("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MidasStochObv[] cacheMidasStochObv = null;

        private static MidasStochObv checkMidasStochObv = new MidasStochObv();

        /// <summary>
        /// Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.
        /// </summary>
        /// <returns></returns>
        public MidasStochObv MidasStochObv(int periodD, int periodK, int smooth)
        {
            return MidasStochObv(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.
        /// </summary>
        /// <returns></returns>
        public MidasStochObv MidasStochObv(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            if (cacheMidasStochObv != null)
                for (int idx = 0; idx < cacheMidasStochObv.Length; idx++)
                    if (cacheMidasStochObv[idx].PeriodD == periodD && cacheMidasStochObv[idx].PeriodK == periodK && cacheMidasStochObv[idx].Smooth == smooth && cacheMidasStochObv[idx].EqualsInput(input))
                        return cacheMidasStochObv[idx];

            lock (checkMidasStochObv)
            {
                checkMidasStochObv.PeriodD = periodD;
                periodD = checkMidasStochObv.PeriodD;
                checkMidasStochObv.PeriodK = periodK;
                periodK = checkMidasStochObv.PeriodK;
                checkMidasStochObv.Smooth = smooth;
                smooth = checkMidasStochObv.Smooth;

                if (cacheMidasStochObv != null)
                    for (int idx = 0; idx < cacheMidasStochObv.Length; idx++)
                        if (cacheMidasStochObv[idx].PeriodD == periodD && cacheMidasStochObv[idx].PeriodK == periodK && cacheMidasStochObv[idx].Smooth == smooth && cacheMidasStochObv[idx].EqualsInput(input))
                            return cacheMidasStochObv[idx];

                MidasStochObv indicator = new MidasStochObv();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.PeriodD = periodD;
                indicator.PeriodK = periodK;
                indicator.Smooth = smooth;
                Indicators.Add(indicator);
                indicator.SetUp();

                MidasStochObv[] tmp = new MidasStochObv[cacheMidasStochObv == null ? 1 : cacheMidasStochObv.Length + 1];
                if (cacheMidasStochObv != null)
                    cacheMidasStochObv.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheMidasStochObv = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MidasStochObv MidasStochObv(int periodD, int periodK, int smooth)
        {
            return _indicator.MidasStochObv(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.
        /// </summary>
        /// <returns></returns>
        public Indicator.MidasStochObv MidasStochObv(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            return _indicator.MidasStochObv(input, periodD, periodK, smooth);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MidasStochObv MidasStochObv(int periodD, int periodK, int smooth)
        {
            return _indicator.MidasStochObv(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// Stochastics of OBV, used for MIDAS analysis, created by MidasExplorer.com.  Used to determine overbought and oversold states.  Can also be used for divergences.  See Midas Technical Analysis by Coles and Hawkins for more details.
        /// </summary>
        /// <returns></returns>
        public Indicator.MidasStochObv MidasStochObv(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MidasStochObv(input, periodD, periodK, smooth);
        }
    }
}
#endregion
